/*-
 * Copyright (c) 2011 Alistair Crooks <agc@NetBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "pstruct.h"

/* get the description from a file */
static int
getdesc(const char *f, char **desc)
{
	struct stat	 st;
	FILE		*fp;

	if ((fp = fopen(f, "r")) == NULL) {
		(void) fprintf(stderr, "can't open descriptor file '%s'\n", f);
		return 0;
	}
	(void) fstat(fileno(fp), &st);
	if ((*desc = calloc(1, st.st_size + 1)) == NULL) {
		(void) fprintf(stderr, "can't alloc for '%s'\n", f);
		(void) fclose(fp);
		return 0;
	}
	if (fread(*desc, 1, st.st_size, fp) != (size_t)st.st_size) {
		(void) fprintf(stderr, "can't read from '%s'\n", f);
		(void) fclose(fp);
		return 0;
	}
	(void) fclose(fp);
	return 1;
}

int
main(int argc, char **argv)
{
	const char	*f;
	uint64_t	 off;
	char		*desc;
	int		 i;

	f = desc = NULL;
	off = 0;
	while ((i = getopt(argc, argv, "d:f:o:")) != -1) {
		switch(i) {
		case 'd':
			desc = optarg;
			break;
		case 'f':
			f = optarg;
			break;
		case 'o':
			off = (uint64_t)strtoll(optarg, NULL, 10);
			break;
		default:
			break;
		}
	}
	if (f && !getdesc(f, &desc)) {
		(void) fprintf(stderr, "%s: can't get descriptor\n", *argv);
		exit(EXIT_FAILURE);
	}
	if (desc == NULL) {
		(void) fprintf(stderr, "%s: no descriptor\n", *argv);
		exit(EXIT_FAILURE);
	}
	if (optind == argc) {
		pstructf(desc, NULL, off);
	}
	for (i = optind ; i < argc ; i++) {
		pstructf(desc, argv[i], off);
	}
	exit(EXIT_SUCCESS);
}
